#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct Config {
    anti_phishing_code: String,

    keymap: Option<String>,
    modules: String,

    auths: Vec<Auth>,

    networks: Vec<Network>,

    #[serde(default)]
    ssh: SSHServer,

    #[serde(default)]
    pre_lvm_crypt: Vec<CryptDev>,
    #[serde(default)]
    lvm: Vec<LvmVG>,
    #[serde(default)]
    crypt: Vec<CryptDev>,

    bootstrap: Bootstrap,
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct Auth {
    name: String,
    #[serde(rename = "sshKey")]
    ssh_key: String,
    password: String,
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct Network {
    name: String,
    interfaces: Vec<NetworkInterface>,
    script: String,
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct NetworkInterface {
    pub var: String,
    pub n: i16,
    pub regexps: Vec<String>,
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct SSHServer {
    listen: String,
    keys: SSHKeys,
}
impl Default for SSHServer {
    fn default() -> Self {
        Self {
            listen: "[::]:22".to_string(),
            keys: SSHKeys::default(),
        }
    }
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct SSHKeys {
    dsa: String,
    rsa: String,
    ecdsa: String,
    ed25519: String,
}
impl Default for SSHKeys {
    fn default() -> Self {
        Self {
            dsa: "id_dsa".to_string(),
            rsa: "id_rsa".to_string(),
            ecdsa: "id_ecdsa".to_string(),
            ed25519: "id_ed25519".to_string(),
        }
    }
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct LvmVG {
    vg: String,
    pvs: LvmPV,

    #[serde(default)]
    defaults: LvmLVDefaults,

    lvs: Vec<LvmLV>,
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct LvmLVDefaults {
    #[serde(default = "default_fs")]
    fs: String,
    raid: Option<Raid>,
}
impl Default for LvmLVDefaults {
    fn default() -> Self {
        Self {
            fs: default_fs(),
            raid: None,
        }
    }
}

fn default_fs() -> String {
    "ext4".to_string()
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct LvmLV {
    name: String,
    #[serde(skip_serializing_if = "Option::is_none")]
    fs: Option<String>,
    #[serde(skip_serializing_if = "Option::is_none")]
    raid: Option<Raid>,
    #[serde(skip_serializing_if = "Option::is_none")]
    size: Option<String>,
    #[serde(skip_serializing_if = "Option::is_none")]
    extents: Option<String>,
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct LvmPV {
    n: i16,
    regexps: Vec<String>,
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct CryptDev {
    name: String,
    dev: Option<String>,
    prefix: Option<String>,
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct Raid {
    mirrors: Option<u8>,
    stripes: Option<u8>,
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct Bootstrap {
    dev: String,
    seed: String,
}
